import chroma from "chroma-js";

const style = getComputedStyle(document.body);

export const COLOR_NAMES = [
	"red",
	"orange",
	"yellow",
	"green",
	"cyan",
	"blue",
	"purple",
	"pink",
] as const;

export type ColorName = (typeof COLOR_NAMES)[number];
export type RGB = [number, number, number];

export class Color {
	constructor(
		private readonly r: number,
		private readonly g: number,
		private readonly b: number,
		private readonly a: number = 1
	) {}

	get hex(): string {
		const rHex = this.r.toString(16).padStart(2, "0");
		const gHex = this.g.toString(16).padStart(2, "0");
		const bHex = this.b.toString(16).padStart(2, "0");

		return `#${rHex}${gHex}${bHex}`;
	}

	get rgb(): string {
		return `rgb(${this.r}, ${this.g}, ${this.b})`;
	}

	get rgba(): string {
		return `rgba(${this.r}, ${this.g}, ${this.b}, ${this.a})`;
	}

	alpha(alpha: number): Color {
		return new Color(this.r, this.g, this.b, alpha);
	}

	static fromName(color: ColorName): Color {
		const rawRgb = style
			.getPropertyValue(`--color-${color}-rgb`)
			.split(", ");

		return new Color(
			parseInt(rawRgb[0], 10),
			parseInt(rawRgb[1], 10),
			parseInt(rawRgb[2], 10)
		);
	}

	static getAll(): Color[] {
		return COLOR_NAMES.map((color) => Color.fromName(color));
	}

	static scale(n: number): Color[] {
		const colors = COLOR_NAMES.map((color) =>
			style.getPropertyValue(`--color-${color}`)
		);

		return chroma
			.scale(colors)
			.mode("lch")
			.colors(n, "rgb")
			.map(([r, g, b]) => new Color(r, g, b));
	}
}
